#!/bin/bash
_dir="$1"
_pwd=`pwd`
_dom='dhms.io'
_fips='true'
_ocsver='4.5'
_artdir="${_pwd}/artifacts"
_kubcfg="${_pwd}/${_dir}/auth/kubeconfig"
_instcfg="${_artdir}/upi-install-config.yaml"
_ovaver="rhcos-${_ocsver}.6-x86_64-vmware"
_install="openshift-install"
_storage="tims-esp"
_cluster="tims-esp"
_portgroup="160_os_ip_01-vds"
_gateway="10.60.0.1"
_mem="64"
_cpu="16"
_occmd='/usr/local/bin/oc'
_boots="${_dir}_bootstrap.ign"
_datas="tims_esp_ssd_02"
_cert="${_artdir}/yaml-ca_certs/apps.${_dir}.${_dom}.crt"
_ckey="${_artdir}/yaml-ca_certs/apps.${_dir}.${_dom}.key"
_ca="${_artdir}/yaml-ca_certs/ti-ca-bundle.crt"
_pass='ryan'
_path="/core/vm/ResourcePools/${_cluster}/${_dir}"
_baseline="1.5-ocs"
_svnrev=`svn info . | grep Revision`
. "${_artdir}/bash_functions"

[ $# -eq 0 ] && { echo "## Usage: $0 version dir"; exit 1; }

rmtimout

[ -d ${_dir} ] && { rm -rf ${_dir}; rm -rf ~/.kube/; }

echo "## deploying baseline ${_baseline} svn rev ${_svnrev}"

echo "## configuring networking"
> /etc/dhcp/dhcpd.${_dir}.conf

echo "## creating dns zones"
perl ${_artdir}/create_zones_from_host.pl ${_dir}.${_dom}

echo "## configuring f5"
perl ${_artdir}/f5_configure_vars.pl ${_dir} > ${_artdir}/f5_configure_vars.yml

ansible-playbook ${_artdir}/f5_configure.yml

echo "## make cluster dir"
mkdir ${_dir}

echo "## pulling lastest installer if not present"
if [ ! -f ${_install} ] ; then
  pullocs
fi

echo "## creating the installation configuration file"
cat ${_instcfg} |\
sed s/FIPS/${_fips}/g |\
sed s/VER/${_dir}/g |\
sed s/DATASTORE/${_datas}/g |\
sed s/DOMAIN/${_dom}/g > ${_dir}/install-config.yaml

echo "## creating the Kuberbetes manifest file"
./${_install} create manifests --dir=${_pwd}/${_dir}/

echo "## rm control plane machines and compute machineSets"
rm -rf ${_pwd}/${_dir}/openshift/99_openshift-cluster-api_master-machines-*.yaml 
rm -rf ${_pwd}/${_dir}/openshift/99_openshift-cluster-api_worker-machineset-*.yaml

echo "## modifying the Kubernets manifest file to prevent Pods form being scheduled on the control plane machines"
perl -p -i -e 's/mastersSchedulable: true/mastersSchedulable: false/g'  ${_dir}/manifests/cluster-scheduler-02-config.yml

echo "## creating the ignition file"
./${_install} create ignition-configs --dir=${_pwd}/${_dir}/

rm -rf /var/www/html/${_boots}
cp ${_dir}/bootstrap.ign /var/www/html/${_boots}
chown apache:apache /var/www/html/*

cat > ${_dir}/append-bootstrap.ign <<EOF
{ "ignition": { "config": { "append": [ { "source": "http://`hostname -i`/${_boots}", "verification": {} } ] },
  "timeouts": {}, "version": "2.1.0" }, "networkd": {}, "passwd": {}, "storage": {}, "systemd": {} }
EOF

echo "## converting the master, worker, and secondary bootstrap Ignition config files to Base64 encoding"
base64 -w0 ${_dir}/master.ign > ${_dir}/master.64
base64 -w0 ${_dir}/worker.ign > ${_dir}/worker.64
base64 -w0 ${_dir}/append-bootstrap.ign > ${_dir}/append-bootstrap.64

echo "## deploying RHCOS machines"
perl ${_artdir}/deploy_core-os.pl --purge --cpu ${_cpu} --mem ${_mem} --dir ${_pwd}/${_dir}/ --template ${_ovaver} --storage ${_storage} --cluster ${_cluster} --portgroup ${_portgroup} --gateway ${_gateway}

echo "## generating cert requests if needed"
if [ "${_dom}" == "dhms.io" ] ; then
  ansible-playbook ${_artdir}/acme_cert-wild.yml --extra-vars "in_subdom=apps.${_dir}"
  _cert="${_artdir}/yaml-ca_certs/${_dom}/*.apps.${_dir}.${_dom}-fullchain.crt"
  _ckey="${_artdir}/yaml-ca_certs/${_dom}/*.apps.${_dir}.${_dom}.key"
fi

echo "## creating the cluster"
./${_install} --dir=${_pwd}/${_dir}/ wait-for bootstrap-complete --log-level=debug

echo "## exporting the KUBECONFIG environment variable"
if [ -f ${_kubcfg} ] 
then
  export KUBECONFIG=${_kubcfg}
else
  echo "## Error, boot strap failed no ${_kubcfg}"
  exit
fi

echo "## checking for CSR after bootstrap"
while [ $(${_occmd} get nodes| grep -i worker | wc -l) -lt "3" ]
do
  signcsrs
done

echo "## monitoring the cluster completion"
./${_install} --dir=${_pwd}/${_dir}/ wait-for install-complete --log-level=debug

echo "## checking for status after install-complete"
checkwaits

echo "## configuring ingress and migrating the router"
${_occmd} project openshift-ingress-operator
${_occmd} patch ingresscontroller/default --type merge -p '{"spec":{"nodePlacement":{"nodeSelector":{"matchLabels":{"node-role.kubernetes.io/infra":""}}}}}'
${_occmd} patch config/cluster --type merge -p '{"spec":{"nodeSelector":{"node-role.kubernetes.io/infra":""}}}'

echo "## checking for status after ingress change"
checkwaits

echo "## adding trust bundles to the proxy"
${_occmd} -n openshift-config create configmap custom-ca --from-file=ca-bundle.crt=${_ca}
${_occmd} patch proxy/cluster --type=merge --patch='{"spec":{"trustedCA":{"name":"custom-ca"}}}'

echo "## replacing the default ingress certificates"
${_occmd} -n openshift-ingress create secret tls custom-ca-certs --cert=${_cert} --key=${_ckey}
${_occmd} -n openshift-ingress-operator patch ingresscontroller.operator default --type=merge -p '{"spec":{"defaultCertificate": {"name": "custom-ca-certs"}}}'

echo "## checking for node status"
nodewait

echo "## adding labels to nodes"
_n=0
for i in `${_occmd} get nodes| grep -i tiaworker | awk {'print $1'}`
do
  if [ $_n -lt "3" ];
  then
    ${_occmd} label node ${i} node-role.kubernetes.io/infra=""
    ${_occmd} label node ${i} node-role.kubernetes.io/worker-
  fi
  ((_n=_n+1))
done

echo "## fixing cloud provider vsphere path"
${_occmd} get cm cloud-provider-config -n openshift-config -o yaml | sed -E "s~folder = \"(.*)\"~folder = \"${_path}\"~g" | ${_occmd} apply -f -

echo "## seting up ocs operator"
${_occmd} create -f ${_artdir}/yaml-ocs/deploy-ocs-operator.yaml

checkwaits
sleep 120

echo "## seting up ocs service"
${_occmd} create -f ${_artdir}/yaml-ocs/deploy-ocs-service.yaml

echo "## adding storage labels to nodes"
for i in `oc get nodes| grep storage | awk {'print $1'}` ;
do
  ${_occmd} label node ${i} cluster.ocs.openshift.io/openshift-storage=""
  ${_occmd} adm taint nodes ${i} node.ocs.openshift.io/storage=true:NoSchedule
done

n=0;
echo "## waiting db-noobaa-db-0 to be Bound"
while ! ${_occmd} get pvc -n openshift-storage 2>&1 | grep db-noobaa-db-0 | grep -q -i Bound ;
do
  ((_n=_n+1))
  sleep 5
done
((_n=_n*5))
echo "## waited ${_n} secs for db-noobaa-db-0 to be Bound"

echo "## configure the internal registry and adding persistent storage to it"
${_occmd} -n openshift-image-registry apply -f ${_artdir}/yaml-image_reg/create-pvc-ocs.yml
${_occmd} patch configs.imageregistry.operator.openshift.io/cluster --type merge -p '{"spec":{"managementState":"Managed"}}'
${_occmd} patch configs.imageregistry.operator.openshift.io/cluster --type merge -p '{"spec":{"storage":{"pvc":{"claim":"image-registry-cephfs"}}}}'

echo "## configuring monitoring the monitoring stack"
${_occmd} create -f ${_artdir}/yaml-monitor/monitor-config-ocs.yaml

echo "## creating the elasticsearch namespace"
${_occmd} create -f ${_artdir}/yaml-logging/openshift-operators-redhat.yaml

checkwaits

echo "## creating RBAC to grant Prometheus permission to access the openshift-operators-redhat Namespace"
${_occmd} create -f ${_artdir}/yaml-logging/rbac-authorization.yaml

echo "## installing the cluster logging operator"
${_occmd} create -f ${_artdir}/yaml-logging/cluster-logging-operator.yaml

checkwaits
sleep 30

echo "## creating a cluster loggin instance"
${_occmd} create -f ${_artdir}/yaml-logging/openshift-logging-ocs.yaml

echo "## checking for status after logging change"
checkwaits

echo "## setting up LDAP authentication" 
${_occmd} -n openshift-config create secret generic ldap-secret --from-literal=bindPassword=${_pass}
${_occmd} apply -f ${_artdir}/yaml-ldap_bind/ldap-oauth.yml

echo "## checking for status after auth change"
checkwaits

echo "## setting up LDAP group sync"
${_occmd} adm groups sync --sync-config=${_artdir}/yaml-ldap_bind/ldap_sync.yml --whitelist=${_artdir}/yaml-ldap_bind/ldap_sync_whitelist.yml --confirm
${_occmd} adm policy add-cluster-role-to-group cluster-admin "CN=core-ose401-admins,OU=core-groups,DC=ti,DC=com"
${_occmd} adm policy add-cluster-role-to-group self-provisioner "CN=core-ose401-users,OU=core-groups,DC=ti,DC=com"

echo "## configuring LDAP group sync"
${_occmd} new-project infra-cronjobs >/dev/null 2>&1
${_occmd} process -f ${_artdir}/yaml-ldap_bind/cronjob-ldap-group-sync-secure.yml | ${_occmd} create -f -

echo "## disabling the self-provisioner role"
${_occmd} patch clusterrolebinding.rbac self-provisioner -p '{"subjects":null}'
${_occmd} patch clusterrolebinding.rbac self-provisioner -p '{"metadata":{"annotations":{"rbac.authorization.kubernetes.io/autoupdate":"false"}}}'
${_occmd} adm policy remove-cluster-role-from-group self-provisioner system:authenticated:oauth

echo "## encrypting etcd"
${_occmd} patch apiserver cluster --type=merge -p '{"spec":{"encryption":{"type":"aescbc"}}}'

echo "## configuring network policy"
${_occmd} -n openshift-config create -f ${_artdir}/yaml-network-policies/project_request_template.yaml
${_occmd} apply -f ${_artdir}/yaml-network-policies/project_config.yaml 

echo "## checking for 'everything' after install"

signcsrs
checkwaits
sleep 120

echo "## setting up jenkins"
cd ${_artdir}/jenkins && ./SetupJenkinsStandalone.sh ${_ocsver}
